-- tb_alu.vhd Version 1.4 by Dr. Jerry H. Tucker
-- Created  9/22/09      
-- entities TB
LIBRARY IEEE;
USE work.all;
USE IEEE.Std_Logic_1164.all; 
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL; 
----------------------------------------------------------
--  ENTITY: TB - Test bench for alu.VHD  Uses only a 2 bit alu
--   NEEDS: alu.vhd
----------------------------------------------------------
ENTITY TB IS
END ENTITY TB;

ARCHITECTURE TEST OF TB IS
    CONSTANT BITS: integer := 2;
    SIGNAL a, b: std_logic_vector((BITS-1) downto 0) := (others => '0');
    signal ai,bi: integer;
    SIGNAL R_add: std_logic_vector((BITS-1) downto 0) := (others => '0');
    signal addi: integer;
    signal add_er: boolean := FALSE;
    SIGNAL ov_add: std_logic := '0';
    SIGNAL R_sub: std_logic_vector(BITS-1 downto 0) := (others => '0');
    signal subi: integer;
    signal sub_er: boolean := FALSE;
    SIGNAL ov_sub: std_logic := '0';
    SIGNAL R_less: std_logic_vector((BITS-1) downto 0) := (others => '0');
    signal less_er: boolean;
    SIGNAL ov_less: std_logic := '0';
    SIGNAL R_and: std_logic_vector((BITS-1) downto 0) := (others => '0');
    signal and_er: boolean;
    SIGNAL ov_and: std_logic := '0';
    SIGNAL R_or: std_logic_vector((BITS-1) downto 0) := (others => '0');
    signal or_er: boolean;
    SIGNAL ov_or: std_logic := '0';
    SIGNAL Z_and, z_or, z_add, z_sub, z_less: std_logic := '0';
BEGIN
   a <= a + 1 after 400 ns;
   b <= b + 1 after 100 ns;
   ai <= conv_integer(signed(a));
   bi <= conv_integer(signed(b)); 
   addi <= ai + bi;
   add_er <= false when (addi = conv_integer(signed(R_add))) else true;
   subi <= ai - bi;
   sub_er <= false when (subi = conv_integer(signed(R_sub))) else true;
   less_er <= false when ((ai < bi) = (r_less = "01"));
   and_er <= false when ((a and b) = r_and) else true;
   or_er <= false when ((a or b) = r_or) else true;
   process
   begin
        wait for 15 ns;
        assert not (add_er and (ov_add = '0'))  report "******* Add error";
        assert not (sub_er and (ov_sub = '0')) report "******* Sub error";
        assert not (less_er and (ov_less = '0')) report "******* Less error";
        assert not and_er report "******* And error";
        assert not or_er report "******* Or error";
        wait on a,b;   
   end process; 
ALUand: entity work.ALU(struc) generic map(BITS)
         PORT MAP (a, b, '0', '0',"00", r_and, Z_and, ov_and);
ALUor: entity work.ALU(struc) generic map(BITS)
         PORT MAP (a,b, '0', '0',"01", r_or, Z_or, ov_or);
ALUadd: entity work.ALU(struc) generic map(BITS)
         PORT MAP (a, b, '0', '0',"10", r_add, Z_add, ov_add);
ALUsub: entity work.ALU(struc) generic map(BITS)
         PORT MAP (a,b, '0', '1',"10", r_sub, Z_sub, ov_sub);
ALUless: entity work.ALU(struc) generic map(BITS)
         PORT MAP (a,b,'0', '1',"11", r_less, Z_less, ov_less);
END ARCHITECTURE TEST;





